//
//  StudentTableViewController.m
//  StudentManager
//
//  Created by Rich on 2/9/11.
//  Copyright 2011 Swarthmore College. All rights reserved.
//

#import "StudentTableViewController.h"


@implementation StudentTableViewController

@synthesize students;
@synthesize detailViewController;

- (StudentDetailViewController *)detailViewController
{
	if (!detailViewController) {
		detailViewController = [[StudentDetailViewController alloc] init];
	}
	return detailViewController;
}

- (NSMutableArray *)students
{
	if (!students) {
		students = [[NSMutableArray alloc] init];
	}
	return students;
}



#pragma mark -
#pragma mark Model

- (NSString *)studentDataPath
{
	NSString *docPath = [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0];
	NSString *filePath = [docPath stringByAppendingPathComponent:@"students.data"];
	return filePath;
}


- (void)loadData
{
	BOOL useFakeData = YES;
	
	NSString *filePath = [self studentDataPath];
	NSMutableArray *loadStudents = [NSKeyedUnarchiver unarchiveObjectWithFile:filePath];
	if (!loadStudents) {
		if (useFakeData) {
			loadStudents = [Student fakeData]; 
		} else {
			loadStudents = [[[NSMutableArray alloc] init] autorelease];
		}
	}
	//[loadStudents sortUsingSelector:@selector(compare:)];
	self.students = loadStudents;
}

- (void)saveData
{
	NSString *filePath = [self studentDataPath];
	BOOL success = [NSKeyedArchiver archiveRootObject:self.students toFile:filePath];
	if (!success) {
		NSLog(@"Error saving data!");
	}
}


#pragma mark -
#pragma mark Initialization


- (id)initWithStyle:(UITableViewStyle)style {
    return [self init];
}


- (id)init
{
	//Call the super class' designated initializer
	if (self = [super initWithStyle:UITableViewStyleGrouped]) {
		[self loadData];
		
		self.navigationItem.leftBarButtonItem = self.editButtonItem;

		UIBarButtonItem *bbi = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemAdd
																			 target:self 
																			 action:@selector(addPressed:)];
		self.navigationItem.rightBarButtonItem = bbi;
		[bbi release];
		
		self.navigationItem.title = @"Students";
	}
	
	return self;
}

#pragma mark -
#pragma mark View lifecycle

/*
- (void)viewDidLoad {
    [super viewDidLoad];

    // Uncomment the following line to display an Edit button in the navigation bar for this view controller.
    // self.navigationItem.rightBarButtonItem = self.editButtonItem;
}
*/


- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
	[self.tableView reloadData];
}

/*
- (void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear:animated];
}
*/
/*
- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
}
*/
/*
- (void)viewDidDisappear:(BOOL)animated {
    [super viewDidDisappear:animated];
}
*/
/*
// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations.
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}
*/


#pragma mark -
#pragma mark Table view data source
/*
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    // Return the number of sections.
    return <#number of sections#>;
}
*/

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    // Return the number of rows in the section.
    return self.students.count;
}


// Customize the appearance of table view cells.
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
    static NSString *CellIdentifier = @"StudentTableViewCell";
    
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {
        cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier] autorelease];
    } 
	
    Student *student = [self.students objectAtIndex:indexPath.row];
	cell.textLabel.text = student.description;
    
    return cell;
}


/*
// Override to support conditional editing of the table view.
- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath {
    // Return NO if you do not want the specified item to be editable.
    return YES;
}
*/



// Override to support editing the table view.
- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath {
    
    if (editingStyle == UITableViewCellEditingStyleDelete) {
        // Delete the row from the data source.
		[self.students removeObjectAtIndex:indexPath.row];
		
        [tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPath] withRowAnimation:UITableViewRowAnimationFade];
    }   
    else if (editingStyle == UITableViewCellEditingStyleInsert) {
        // Create a new instance of the appropriate class, insert it into the array, and add a new row to the table view.
    }   
}




// Override to support rearranging the table view.
- (void)  tableView:(UITableView *)tableView 
 moveRowAtIndexPath:(NSIndexPath *)fromIndexPath 
		toIndexPath:(NSIndexPath *)toIndexPath {
	Student *s = [students objectAtIndex:fromIndexPath.row];
	[s retain];
	
	[students removeObjectAtIndex:fromIndexPath.row];
	[students insertObject:s atIndex:toIndexPath.row];
	[s release];
}



/*
// Override to support conditional rearranging of the table view.
- (BOOL)tableView:(UITableView *)tableView canMoveRowAtIndexPath:(NSIndexPath *)indexPath {
    // Return NO if you do not want the item to be re-orderable.
    return YES;
}
*/


- (void)addPressed:(id)sender
{
	Student *newStudent = [[Student alloc] init];
	[self.students addObject:newStudent];
	self.detailViewController.editStudent = newStudent;
	[newStudent release];
    [self.navigationController pushViewController:self.detailViewController animated:YES];
    self.detailViewController = nil;
}


#pragma mark -
#pragma mark Table view delegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {

	self.detailViewController.editStudent = [self.students objectAtIndex:indexPath.row];
    [self.navigationController pushViewController:self.detailViewController animated:YES];
    self.detailViewController = nil;
	
}


#pragma mark -
#pragma mark Memory management

- (void)didReceiveMemoryWarning {
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Relinquish ownership any cached data, images, etc. that aren't in use.
}

- (void)viewDidUnload {
    // Relinquish ownership of anything that can be recreated in viewDidLoad or on demand.
    // For example: self.myOutlet = nil;
}


- (void)dealloc {
	[students release];
    [super dealloc];
}


@end

